package main

import (
	"context"
	"os"
	"os/signal"

	"github.com/sirupsen/logrus"

	"git.janky.solutions/finn/go-project-template/config"
	"git.janky.solutions/finn/go-project-template/db"
	"git.janky.solutions/finn/go-project-template/web"
)

func main() {
	run()
}

func run() {
	if err := config.Load(context.Background()); err != nil {
		logrus.WithError(err).Fatal("error loading config")
	}

	ctx, cancel := signal.NotifyContext(context.Background(), os.Interrupt)
	defer cancel()

	if err := db.Migrate(ctx); err != nil {
		logrus.WithError(err).Fatal("error migrating database")
	}

	go web.ListenAndServe()

	<-ctx.Done()

	if err := web.Shutdown(ctx); err != nil {
		logrus.WithError(err).Error("error shutting down web server")
	}
}
