package config

import (
	"context"
	"errors"
	"fmt"
	"math/rand"

	"git.janky.solutions/finn/conflib"
	"github.com/sirupsen/logrus"
)

type Config struct {
	Database string
	Web      WebConfig
	OAuth2   OAuth2
	LogLevel string `default:"INFO"`
}

type WebConfig struct {
	Bind       string `default:":8080"`
	SessionKey string
	BaseURL    string
}

var C Config

func Load(ctx context.Context) error {
	logrus.Info("loading config file")

	var err error
	C, err = conflib.Load[Config]("/etc/go-project-template/go-project-template.json", "go-project-template.json")
	if err != nil {
		return err
	}

	level, err := logrus.ParseLevel(C.LogLevel)
	if err != nil {
		return fmt.Errorf("error parsing requested log level %s: %v", C.LogLevel, err)
	}
	logrus.SetLevel(level)

	if C.Web.SessionKey == "" {
		logrus.Info("No session key specified. Here's a good one if you need it: ", generateSessionKey())
		return errors.New("session key may not be empty")
	}

	if err := C.OAuth2.Load(ctx); err != nil {
		return err
	}
	return nil
}

var letterRunes = []rune("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890")

func generateSessionKey() string {
	b := make([]rune, 64)
	for i := range b {
		b[i] = letterRunes[rand.Intn(len(letterRunes))]
	}
	return string(b)
}
